/**
 * @(#) WASExp6_0fpLnxOnPwrPDC.java
 * 
 * Licensed Materials - Property of IBM
 * 
 * 5724-J10
 * 
 * (C) Copyright IBM Corp. 2007
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 * This is sample code made available for use in accordance with terms set 
 * forth in the license agreement document for the IBM Express Runtime.
 */
package com.ibm.iru.wasexp6_0fp_lnxonpwr;

import com.ibm.iru.message.NLSKeys;
import com.ibm.jsdt.support.SupportLinuxBase;
import com.ibm.jsdt.support.SupportLinuxHelper;
import java.io.File;
import java.io.IOException;

/**
 * This class checks if fix pack install is required.
 */
public class WASExp6_0fpLnxOnPwrPDC extends SupportLinuxBase
{
  private static final String copyright0 = "Licensed Materials - Property of IBM";
  private static final String copyright1 = "5724-J10";
  private static final String copyright2 = "(C) Copyright IBM Corp. 2006 All Rights Reserved.";
  private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
  private SupportLinuxHelper ivHelper;
  private String ivInstallDir = null;
  //WebSphere major version you are installing fix pack for.
  private String WAS_MAJOR_VERSION = "6.0.2";

  /**
   *  Initialize variables
   */
  public void init()
  {
    ivHelper = getLinuxHelper();
    // set the message bundle and jar file for the resource strings
    setMainResources("com.ibm.iru.message.MessagesNLS");
    setVariableName("installDir");
    ivInstallDir = ivHelper.getIbmNsiPropValue(this);
  }

  /**
   * Check if WAS version 6.0.2.x is installed in the specified location.
   * 
   * @return - PDC_EXIST if 6.0.2.17 is an installed version
   *         - FAILURE if installed version is less than 6.0.2 or is 6.1 or greater. 
   *         - PDC_DOES_NOT_EXIST otherwise
   */
  private int determineInstallAction()
  {
    int returnCode = FAILURE;
    String installedVersion = null;
    
    setFileName(ivInstallDir);
    if (!ivHelper.fileExists(this))
    {
      setMessage(getResourceString(NLSKeys.DIRECTORY_NOT_FOUND, ivInstallDir));
      ivHelper.postErrorMessageToDeployerUI(this);
      returnCode = FAILURE;
    }
    
    setPath(ivInstallDir);
    String UNKNOWN_VERSION_STRING = "UNKNOWN_VERSION";
    String[] wasVersionInfo = new String[] { null, null };
    setFileName(ivInstallDir);
    if (ivHelper.fileExists(this))
    {
      // get installed WAS version
      File file = new File(ivInstallDir);
      if ((file.list()).length != 0)
      {
        // check WebSphere Application Server - Express version. Store the file ending with .product.
        File WASVersionDir = new File(ivInstallDir + SLASH + "properties" + SLASH + "version");
        String versionDir = ivInstallDir + SLASH + "properties" + SLASH + "version";
        String[] filteredCmdFiles = directoryGrep(versionDir, ".product");
        if (filteredCmdFiles != null && filteredCmdFiles.length > 0)
        {
          // there should only be one .product file
          setFileName(WASVersionDir.getAbsolutePath() + SLASH + filteredCmdFiles[0]);
          if (ivHelper.fileExists(this))
          {
            try
            {
              String s = ivHelper.readFile(this);
              String[] version = s.split("</?version>");
              if (version.length > 1)
              {
                wasVersionInfo[0] = version[1];
              }
              else
              {
                wasVersionInfo[0] = UNKNOWN_VERSION_STRING;
              }
              String[] id = s.split("</?id>");
              if (id.length > 1)
              {
                wasVersionInfo[1] = id[1];
              }
              else
              {
                wasVersionInfo[1] = UNKNOWN_VERSION_STRING;
              }
            }
            catch (IOException e)
            {
              // log message
              String[] ioOutput = { e.toString() };
              setMessage(getResourceString(NLSKeys.CMD_EXCEPTION, ioOutput));
              ivHelper.log(this);
              return FAILURE;
            }
          }
        }
        else
        {
          // No .product file was found in the target directory
          setMessage(getResourceString(NLSKeys.FILE_NOT_FOUND, new String []{WASVersionDir.getAbsolutePath() + File.separator + "WAS.product"}));
          ivHelper.log(this);  
          return FAILURE;
        }
      }
    }
    // determine installation acation
    if (wasVersionInfo != null)
    {
      installedVersion = wasVersionInfo[0];
      String installedType = wasVersionInfo[1];
      if (!(installedType.toUpperCase().equals("EXPRESS")) || (installedVersion.equals("UNKNOWN_VERSION")))
      {
        setMessage(getResourceString(NLSKeys.WAS_NON_EXPRESS_VERSION));
        ivHelper.postErrorMessageToDeployerUI(this);
        returnCode = FAILURE;
      }
      else
      {
        //compare installed version with the verson to be installed
        setVersion(installedVersion);
        setCompareVersion(ivHelper.getProductInstallingVersion(this));
        switch (ivHelper.compareVersions(this))
        {
          //version is lower than the version to be installed
          case -1:
          {
            //check if version is 6.0.2.x
            if (trimVersionToNVersionPlaces(3, installedVersion).equalsIgnoreCase(WAS_MAJOR_VERSION))
            {
              //continue installation
              returnCode = PDC_DOES_NOT_EXIST;
            }
            else
            {
              //version is not 6.0.2.x, we can not apply fix pack
              setMessage(getResourceString(NLSKeys.DIFFERENT_VERSION,
                  new String[] { installedVersion, ivHelper.getProductInstallingVersion(this) }));
              ivHelper.postErrorMessageToDeployerUI(this);
              returnCode = FAILURE;
            }
            break;
          }
          //versions are equal
          case 0:
          {
            returnCode = PDC_EXISTS;
            break;
          }
          //later version already installed - no need to continue installation
          case 1:
          {
            setMessage(getResourceString(NLSKeys.DIFFERENT_VERSION,
                new String[] { installedVersion, ivHelper.getProductInstallingVersion(this) }));
            ivHelper.postErrorMessageToDeployerUI(this);
            returnCode = FAILURE;
            break;
          }
        }
      }
    }
    return returnCode;
  }

  /**
   * Concatenates the installed version to the number of decimals specified using the '.' as a separator. For example, if the
   * installed version is 6.0.2.1 and the versionPlaces = 3, then 6.0.2 would be returned.
   * 
   * @return The installed version number trimmed to the number of places specified.
   */
  public String trimVersionToNVersionPlaces(int versionPlaces, String fullVersion)
  {
    int startIndex = 0;
    for (int i = 0; i < versionPlaces; ++i)
    {
      int nextIndex = fullVersion.indexOf(".", startIndex + 1);
      if (nextIndex == -1)
      {
        return fullVersion;
      }
      else
      {
        startIndex = nextIndex;
      }
    }
    return fullVersion.substring(0, startIndex);
  }

  /**
   * Reads the file and directory names and returns a list of
   * names that matches the regular expresion. For example, if the regular
   * expression was "*.exe", then all file names ending in".exe" are
   * returned in the array.This is not a recursive grep and only
   * the directory name is searched. See the java.util.regex.Pattern javadoc
   * for regular expression details.
   * 
   * @param directoryName -
   *            The directory name should contain the absolute path.
   * @param regularExpression -
   *            The regular expression to match.
   * @return Array of names that match the regular expression, a zero length
   *         array is returned if no matches are found or the directory is
   *         empty. Null is returned if the directory doesnot exist.
   */

  public String[] directoryGrep(String directoryName, String regularExpression)
  {
    String[] filteredFiles = null;
    File fileDir = new File(directoryName);
    setFileName(directoryName);
    if (ivHelper.fileExists(this))
    {
      // store the file matching the regular expression
      filteredFiles = fileDir.list(new FileFilter(regularExpression));
    }
    return filteredFiles;
  }
  
  /**
   * MAIN 
   */
  public static void main(String[] args)
  {
    WASExp6_0fpLnxOnPwrPDC pdc = new WASExp6_0fpLnxOnPwrPDC();
    pdc.init();
    System.exit(pdc.determineInstallAction());
  }
}